( cd usr/lib64 ; rm -rf libopenobex.so.1 )
( cd usr/lib64 ; ln -sf libopenobex.so.1.3.0 libopenobex.so.1 )
( cd usr/lib64 ; rm -rf libopenobex.so )
( cd usr/lib64 ; ln -sf libopenobex.so.1.3.0 libopenobex.so )
